@@ -2,7 +2,7 @@
 //
 // Matrix.cpp: Rcpp R/C++ interface class library -- Matrix unit tests
 //
-// Copyright (C) 2013 - 2014    Dirk Eddelbuettel, Romain Francois and Kevin Ushey
+// Copyright (C) 2013 - 2016  Dirk Eddelbuettel, Romain Francois and Kevin Ushey
 //
 // This file is part of Rcpp.
 //
@@ -99,6 +99,12 @@ double runit_NumericMatrix_row( NumericMatrix m){
     return std::accumulate( first_row.begin(), first_row.end(), 0.0 ) ;
 }
 
+// [[Rcpp::export]]
+double runit_NumericMatrix_row_const( const NumericMatrix m){
+    NumericMatrix::ConstRow first_row = m.row(0) ;
+    return std::accumulate( first_row.begin(), first_row.end(), 0.0 ) ;
+}
+
 // [[Rcpp::export]]
 std::string runit_CharacterMatrix_row( CharacterMatrix m ){
     CharacterMatrix::Row first_row = m.row(0) ;
@@ -108,6 +114,15 @@ std::string runit_CharacterMatrix_row( CharacterMatrix m ){
     return res ;
 }
 
+// [[Rcpp::export]]
+std::string runit_CharacterMatrix_row_const( const CharacterMatrix m ){
+    CharacterMatrix::ConstRow first_row = m.row(0) ;
+    std::string res(
+    	std::accumulate(
+    		first_row.begin(), first_row.end(), std::string() ) ) ;
+    return res ;
+}
+
 // [[Rcpp::export]]
 IntegerVector runit_GenericMatrix_row( GenericMatrix m ){
     GenericMatrix::Row first_row = m.row(0) ;
@@ -119,12 +134,29 @@ IntegerVector runit_GenericMatrix_row( GenericMatrix m ){
     return out ;
 }
 
+// [[Rcpp::export]]
+IntegerVector runit_GenericMatrix_row_const( const GenericMatrix m ){
+    GenericMatrix::ConstRow first_row = m.row(0) ;
+    IntegerVector out( first_row.size() ) ;
+    std::transform(
+    	first_row.begin(), first_row.end(),
+    	out.begin(),
+    	unary_call<SEXP,int>( Function("length" ) ) ) ;
+    return out ;
+}
+
 // [[Rcpp::export]]
 double runit_NumericMatrix_column( NumericMatrix m ){
     NumericMatrix::Column col = m.column(0) ;
     return std::accumulate( col.begin(), col.end(), 0.0 ) ;
 }
 
+// [[Rcpp::export]]
+double runit_NumericMatrix_column_const( const NumericMatrix m ){
+    NumericMatrix::ConstColumn col = m.column(0) ;
+    return std::accumulate( col.begin(), col.end(), 0.0 ) ;
+}
+
 // [[Rcpp::export]]
 NumericMatrix runit_NumericMatrix_cumsum( NumericMatrix input ){
     int nr = input.nrow(), nc = input.ncol() ;
@@ -147,6 +179,15 @@ std::string runit_CharacterMatrix_column( CharacterMatrix m){
     return res ;
 }
 
+// [[Rcpp::export]]
+std::string runit_CharacterMatrix_column_const( const CharacterMatrix m){
+    CharacterMatrix::ConstColumn col = m.column(0) ;
+    std::string res(
+        std::accumulate( col.begin(), col.end(), std::string() )
+    ) ;
+    return res ;
+}
+
 // [[Rcpp::export]]
 IntegerVector runit_GenericMatrix_column( GenericMatrix m ){
     GenericMatrix::Column col = m.column(0) ;
@@ -159,6 +200,18 @@ IntegerVector runit_GenericMatrix_column( GenericMatrix m ){
     return wrap(out) ;
 }
 
+// [[Rcpp::export]]
+IntegerVector runit_GenericMatrix_column_const( const GenericMatrix m ){
+    GenericMatrix::ConstColumn col = m.column(0) ;
+    IntegerVector out( col.size() ) ;
+    std::transform(
+    	   col.begin(), col.end(),
+    	   out.begin(),
+    	   unary_call<SEXP,int>( Function("length" ) )
+    ) ;
+    return wrap(out) ;
+}
+
 // [[Rcpp::export]]
 List runit_Row_Column_sugar( NumericMatrix x){
     NumericVector r0 = x.row(0) ;
@@ -258,3 +311,23 @@ NumericMatrix transposeNumeric(const NumericMatrix & x) {
 CharacterMatrix transposeCharacter(const CharacterMatrix & x) {
     return transpose(x);
 }
+
+// [[Rcpp::export]]
+NumericMatrix matrix_scalar_plus(const NumericMatrix & x, double y) {
+    return x + y;
+}
+
+// [[Rcpp::export]]
+NumericMatrix matrix_scalar_plus2(const NumericMatrix & x, double y) {
+    return y + x;
+}
+
+// [[Rcpp::export]]
+NumericMatrix matrix_scalar_divide(const NumericMatrix & x, double y) {
+    return x / y;
+}
+
+// [[Rcpp::export]]
+NumericMatrix matrix_scalar_divide2(const NumericMatrix & x, double y) {
+    return y / x;
+}
